package com.jcg;

import com.google.gson.Gson;
import com.google.gson.internal.LinkedTreeMap;
import com.google.gson.reflect.TypeToken;
import org.apache.http.HttpHost;
import org.elasticsearch.action.bulk.BulkProcessor;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.bulk.BulkResponse;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestClient;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.client.core.CountRequest;
import org.elasticsearch.client.core.CountResponse;
import org.elasticsearch.index.query.QueryBuilders;

import java.io.BufferedReader;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

public class ElasticSearchHighLevelBulkRestClient {
    private static final Path flightDataJsonFile =
            Paths.get("src/main/resources/flightdata/2016-07-01-1300Z.json");

    public static void main(String[] args) {
        final RestHighLevelClient restClient = new RestHighLevelClient(
                RestClient.builder(new HttpHost("localhost", 9200, "http")));

        BulkProcessor.Listener listener = new BulkProcessor.Listener() {
            @Override
            public void beforeBulk(long executionId, BulkRequest request) {
                int numberOfActions = request.numberOfActions();
                System.out.printf("Executing bulk [%d] with %d requests\n",
                        executionId, numberOfActions);
            }

            @Override
            public void afterBulk(long executionId, BulkRequest request,
                                  BulkResponse response) {
                if (response.hasFailures()) {
                    System.err.printf("Bulk [%d] executed with failures\n", executionId);
                } else {
                    System.err.printf("Bulk [%d] completed in %d milliseconds\n",
                            executionId, response.getTook().getMillis());
                }
            }

            @Override
            public void afterBulk(long executionId, BulkRequest request,
                                  Throwable failure) {
                System.err.println("Failed to execute bulk process: " + failure + "\n");
            }
        };

        BulkProcessor bulkProcessor = BulkProcessor.builder(
                (request, bulkListener) ->
                        restClient.bulkAsync(request, RequestOptions.DEFAULT, bulkListener),
                listener).build();
        addDocumentsToIndex("flights", bulkProcessor);
        try {
            boolean terminated = bulkProcessor.awaitClose(30L, TimeUnit.SECONDS);
            while (!terminated) {
                terminated = bulkProcessor.awaitClose(30L, TimeUnit.SECONDS);
                Thread.sleep(1000);
            }
            bulkProcessor.close();
            System.out.println("/flight/_count:" + countDocuments("flights", restClient));
            restClient.close();
        } catch (InterruptedException | IOException e) {
            e.printStackTrace();
        }
    }

    @SuppressWarnings("unchecked")
    private static void addDocumentsToIndex(final String index, BulkProcessor bulkProcessor) {
        final Gson gson = new Gson();
        try (BufferedReader reader = Files.newBufferedReader(flightDataJsonFile)) {
            Map<String, Object> map = gson.fromJson(reader,
                    new TypeToken<Map<String, Object>>() {
                    }.getType());
            List<Object> acList = (List<Object>) (map.get("acList"));
            for (Object item : acList) {
                LinkedTreeMap<String, Object> flight = (LinkedTreeMap<String, Object>) item;
                final long id = Double.valueOf(flight.get("Id").toString()).longValue();
                flight.remove("Id");
                bulkProcessor.add(new IndexRequest(index)
                        .id(String.valueOf(id))
                        .source(flight));
            }
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static long countDocuments(String index, RestHighLevelClient restClient) {
        long numOfDocuments = 0;
        CountRequest countRequest = new CountRequest(index);
        countRequest.query(QueryBuilders.matchAllQuery());
        try {
            CountResponse countResponse = restClient
                    .count(countRequest, RequestOptions.DEFAULT);
            numOfDocuments = countResponse.getCount();
        } catch (IOException e) {
            e.printStackTrace();
        }
        return numOfDocuments;
    }

}
