package com.jcg;

import java.io.File;
import java.io.IOException;
import java.nio.file.DirectoryStream;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.Arrays;
import java.util.Comparator;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * Converts a flight data json file to a format that can be imported to ElasticSearch
 * using its bulk API.
 */
public class JsonFlightFileConverter {

    public static void main(String[] args) {
        if (args.length == 1) {
            Path inDirectoryPath = Paths.get(args[0]);
            if (inDirectoryPath != null) {
                Path outDirectoryPath = Paths.get(inDirectoryPath.toString(), "out");
                try {
                    if (Files.exists(outDirectoryPath)) {
                        Files.walk(outDirectoryPath)
                                .sorted(Comparator.reverseOrder())
                                .map(Path::toFile)
                                .forEach(File::delete);
                    }
                    Files.createDirectory(Paths.get(inDirectoryPath.toString(), "out"));
                } catch (IOException e) {
                    e.printStackTrace();
                }

                try (DirectoryStream<Path> ds = Files.newDirectoryStream(inDirectoryPath, "*.json")) {
                    for (Path inFlightDataJsonFile : ds) {
                        String result = "";
                        try (Stream<String> stream = Files.lines(inFlightDataJsonFile.toAbsolutePath())) {
                            result = stream.parallel()
                                    .map(line -> line.split("\\{"))
                                    .flatMap(Arrays::stream)
                                    .skip(3)
                                    .map(s -> createResult(s))
                                    .collect(Collectors.joining());
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                        result = result.substring(0, result.lastIndexOf("]")) + "\n";
                        Path outFlightDataJsonFile = Paths.get(outDirectoryPath.toString(), inFlightDataJsonFile.getFileName().toString());
                        Files.createFile(outFlightDataJsonFile);
                        Files.writeString(outFlightDataJsonFile, result);
                    }
                } catch (IOException ex) {
                    ex.printStackTrace();
                }
            }
        } else {
            System.out.println("Usage: java JsonFlightFileConverter <directory of json files>");
        }
    }

    /**
     * Create the result required by ElasticSearch's bulk API
     *
     * @param s a flight data JSON object
     * @return the flight data JSON object prepended by {@code {"index":{"_id":"<id>"}}}
     */
    private static String createResult(String s) {
        final String[] keyValues = s.split(",");
        if (keyValues != null && keyValues.length > 0) {
            final String[] idKeyValue = keyValues[0].split(":");
            if (idKeyValue != null && idKeyValue.length > 1)
                return "{\"index\":{\"_id\":" + idKeyValue[1] + "}}\n" + "{" + removeLastComma(removeIdEntry(s)) + "\n";
        }
        return "";
    }

    /**
     * Removes {@code "Id":<id>,}
     *
     * @param s a flight document (JSON object)
     * @return a flight document without its {@code Id} entry
     */
    private static String removeIdEntry(String s) {
        return s.substring(s.indexOf(",") + 1);
    }

    private static String removeLastComma(String s) {
        if (s.isEmpty()) return "";
        return s.charAt(s.length() - 1) == ',' ? s.substring(0, s.length() - 1) : s;
    }
}
